
# q-expansion of newform 54.10.a.c, downloaded from the LMFDB on 15 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 54
weight = 10
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[16], [0], [-435], [-2527], [-9123], [-79180], [-437976], [116966], [-261102], [-396150], [-5882533], [-8986246], [17449566], [-32094646], [20965782], [40669047], [-84383076], [-148038424], [154939106], [168343560], [418697993], [210598040], [776394525], [370837746], [309841967], [1759656243], [-13340284], [-102477015], [-648714310], [-3072510498], [-3341088457], [-4850646159], [3900207882], [-5343650476], [-5940275907], [1247792159], [5124441548], [11791041428], [-10407520038], [-3503697279], [-18951497865], [20293182128], [-11916518100], [-5029677115], [-595780425], [19469445527], [-20944892914], [-37424029336], [-15387120636], [-71395420642], [43917720174], [76954410006], [-22709590234], [-82727612352], [87979033068], [-150737244342], [68618106450], [23568610745], [122854756952], [34916354400], [-76164523414], [177323241894], [191554414760], [-152904586782], [-12380043715], [60294787923], [-74503463110], [-192660804358], [-528835016253], [-194816060674], [84392335506], [-249682624542], [-553324605937], [-130552201240], [-321464672404], [-680222558784], [794586957717], [197912708156], [95647531956], [282203274359], [152503073196], [-107657349064], [-77746172814], [120952428365], [1031931239699], [-620515285548], [751039725258], [1333703540783], [-527899214283], [1087456159469], [801185798709], [389798185206], [-1909920932368], [388418035911], [-51804378430], [-991895125050], [2746424272191], [-516949403148], [-312201985336], [-1355334038620], [-2769178568848], [-890708504283], [-22748687229], [-4337724304932], [2480770822556], [-1503867175018], [-1807668446307], [-4130047075986], [1623234005862], [1879304719187], [543492011936], [3906380851454], [-4164497286222], [2429658250292], [-1520975396455], [609689216766], [4952410904900], [3202851084048], [-6197499064695], [8010193413243], [-8948169526282], [5325547856141], [110212287654], [3484628918460], [-11016948098860], [-8818969538745], [-1849141437748], [6421345328484], [-5649199701487], [-1243093115374], [-2209415205544], [166985814228], [-4970597135065], [9481779799634], [-9470647619904], [-10272748548031], [11330495814258], [6869157362912], [8358674543097], [-5370093519282], [6153955234166], [7737432482586], [21959497712063], [1897610654016], [11462453870258], [-24048961162944], [22855857036806], [-26176420888152], [-3561672813640], [-27356378949828], [-11051878281334], [-27600694262100], [298888822262], [30854466943044], [-23374577232322], [-27311058283578], [-28851269282701], [4944599409180], [-32856291352357], [46051709132913], [-28944029501853], [24910536235572], [39841413248009], [-17695294230843], [-3245154290742], [41761516274166], [-57217399700623], [-4420271475844], [43373016696251], [40714873901718], [-25026545675121], [-33922469862538], [-68765086071714], [69652439757578], [49269640508975], [-25475134705698], [-36099760359490], [80025859111689], [73985828885201], [44678113172408], [-80288370754009], [45388951704129], [67480133137382], [36751141806444], [49283217068100], [62185688226561], [-79636302364642], [14583209436848], [-76063614121573], [20467043527554], [-26590765887214], [-46834770332265], [-82006838258866], [12630650215881], [-70871650162596], [-128616370171500], [-73656095753113], [111396932266142], [-120097615955028], [47255848610868], [84101669168361], [42416381732087], [-66339262035880], [73189730853245], [-20220601565817], [98195916289266], [-6189634649140], [91389119627589], [-145306522776210], [74983724278112], [113867990527529], [73851277957674], [60936413169485], [158730222500283], [117375659172312], [14258938588697], [59642827033232], [-94727422818042], [35712386202114], [205450187196246], [-128268993149962], [-135758596594876], [-199473133318824], [85268006629580], [236692567547241], [-31839653537656], [-150673941130056], [244599112747170], [-174835893701464], [-95982752190588], [-239062949124370], [80101524234872], [-112652104863688], [-49954520612604], [203970405185354], [324307293171276], [-181065197709970], [528021808110], [198786368888439], [13095797990724], [123458562124140], [48998933563970], [107309369314721], [53215366624472], [-295610137876560], [161721662406924], [-85383299965669], [-388778252516391], [-276075359480662], [-272273583626388], [413893168520420], [-336987948882672], [-190228422816930], [172922120381441], [118812177080283], [152455600103121], [-123509127343516], [64115812676072], [241510208336343], [-55517017885711], [-611005606330543], [-122414368672869], [-69359217105562], [-70294133835448], [-156966434169420], [244800225200972], [637487363572434], [-221002323075150], [-452572730196388], [450105099911862], [-110863738075336], [-417854097937312], [-2880314299627], [-54813680954971], [767476933525361], [-464484828312529], [744346781954493], [-396940956410176], [59186109206927], [-666250890567468], [241546940689668], [-434037583689325], [-442039382065878], [662931784624034], [83100985991606], [-187367610485070], [-664553222832589], [822667078411917], [765853875601616], [899059418478144], [-272610543242058], [-53392384710387], [391372254052620], [-110739773085717], [-108480488769250], [86873768378430], [667858610927999], [163626471882471], [-272073306076047], [-146742050257738], [-978310951123462], [618237116863893], [-1248458028606007], [141088151523741], [-495415913293114], [-276755172687457], [375412019462940], [873516010052558], [-1378177349606184], [-463738393598344], [-1026353641856172], [-559934011001961], [388898675271894], [-1648197321044884], [185421701769768], [1224285542070122], [1071769884413385], [17062782498420], [-790550375394958], [-1643605854300174], [661965615400124], [234555639172301], [-1092245625309285], [1305142099387445], [1352985328443576], [823612465954415], [307092527346998], [-1314855557168200], [1191652251947400], [29175874710129], [-452046083085430], [-1816710497956218], [1967829726560303], [529653582651576], [954788517505640], [1211483227179465], [1304005069879850], [1001459136377670], [1846393564311353], [-489422648812003], [-342902915872414], [1051017507889824], [-1121903934535389], [1611601928151608], [-1149104072503761], [2492174113382556], [-1334550580155214], [-195029918519110], [1458699950308818], [-1016302084970022], [-1470816104330752], [-2004607941415189], [1693354257235806], [528959753150375], [-25525093851250], [-1090893455430138], [944279581183782], [-2329005078548001], [3125526055169478], [1857269473345146], [1438657938273296], [-2388722364432384], [1082085017380020], [-834049514062977], [568637688994814], [-1988423416597858], [-3360498997161102], [-3433783663541437], [2622576221856857], [-1662613564648011], [828225934632812], [2218855350205014], [-3074613334775169], [3082836050525999], [681050582823836], [-600053676230784], [-1691673294370050], [1247740587042059], [1661810385251286], [2133644524522670], [-219792307690347], [1762515911553234], [23702687870312], [4433394726174648], [4910544144136460], [-4034570646117474], [-2467178449235332], [-2111812008907228], [1308970958879774], [-1656093013295772], [610333355000225], [1635364222925646], [5103696261238236], [-2650060871834218], [-1681995460055964], [-314723039648290], [-4106019944398336], [1098751443779190], [-1726340664259618], [-3648427094725743], [-3320519786686780], [-4025176057152126], [-3299902710457048], [1580803510369560], [4474266911627427], [-3682416541613605], [-117851150063158], [3816565997694420], [2980098462278324], [79548159548235], [3461148261533279], [-643665503878935], [103086400827105], [-4364878452200332], [-3696493516151479], [-2143577386373409], [4864784235481890], [3176763635533736], [477077617998270], [-866845268793894], [4305346099230594], [-942993572915734], [756651296745228], [-5119390693306119], [2620366878584981], [7991048468182119], [-2238080264354535], [5812034694254658], [1561538915998040], [-1370192753413140]]
